@@ -31,24 +31,31 @@ sub register {
 
   $app->hook(
     before_dispatch => sub {
-      my $c = shift;
+      my $tx = $_[0]->tx;
+      my ($req, $timeout, $url);
 
-      $c->tx->req->env->{t0} = [gettimeofday];
-      $c->tx->on(
+      $tx->req->env->{t0} = [gettimeofday];
+
+      if (my $stream = Mojo::IOLoop->stream($tx->connection)) {
+        $stream->on(timeout => sub { $timeout = 1 });
+      }
+
+      $tx->on(
         finish => sub {
-          my $tx  = shift;
-          my $req = $tx->req;
-          my $url = $req->url->clone->to_abs;
+          my $tx   = shift;
+          my $code = $tx->res->code;
+
+          $code ||= 504 if $timeout;
+          $code or return;
+          $req = $tx->req;
+          $url = $req->url->clone->to_abs->userinfo(undef);
 
-          $url->userinfo(undef);
           unshift @{$url->path->parts}, @{$url->base->path->parts};
 
           $log->info(
             sprintf '%s %s %s %s %.4fs',
             $req->env->{identity} || $tx->remote_address,
-            $req->method, $url,
-            $tx->res->code || '000',
-            tv_interval($req->env->{t0}),
+            $req->method, $url, $code, tv_interval($req->env->{t0}),
           );
         }
       );
